unit ASICamcode;

interface

uses
  // Instances,
  Windows,
  Messages,
  MMSystem,
  Shellapi,
  ShlObj,
  SysUtils,
  Variants,
  Classes,
  IniFiles,
  Math,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ComCtrls,
  Menus,
  StdCtrls,
  ExtCtrls,
  ExtDlgs,
  // Helper,
  ASICamera2,
  Photo;

// =============================================================================

const
  TAB_PREVIEW = 0;
  TAB_CAPTURE = 1;
  TAB_STACK = 2;

  MAX_CAM_CONTROLS = 32;

  IMG_TYPE_RAW8 = 1;
  IMG_TYPE_RGB24 = 2;
  IMG_TYPE_RAW16 = 4;
  IMG_TYPE_Y8 = 8;

type
  TResolution = record
    Width: Integer;
    Height: Integer;
  end;

const
  CAM_RESOLUTION: array [0 .. 4] of TResolution = ((Width: 640; Height: 480),
    (Width: 1280; Height: 960), (Width: 960; Height: 960), (Width: 480;
    Height: 480), (Width: 1280; Height: 720));

  // =============================================================================

type
  TCamStatus = (csNoCamera, csClosed, csOpened, csCapturing, csError,
    csUnknown);

type
  PASIControlCapsArray = ^TASIControlCapsArray;
  TASIControlCapsArray = array [0 .. Pred(MAXWORD)] of TASI_CONTROL_CAPS;

type
  TfrmASICam = class;

  TCaptureThread = class(TThread)
  private
    FOwner: TfrmASICam;
    FActivePage: Integer;
    FControlCaps: array [0 .. MAX_CAM_CONTROLS - 1] of TASI_CONTROL_CAPS;
    FCamInfo: TASI_CAMERA_INFO;
    FCameraID: Integer;
    FCamStatus: TCamStatus;
    FCamModel: AnsiString;
    FImgTypes: Integer;
    FCapturedFrame: Pointer;
    FCapturedFrameSize: Cardinal;
    FGain: Integer;
    FExposure: Integer;
    FExposureRange: Integer;
    FFrameCount: Integer;
    FGamma: Integer;
    FUSBTraffic: Integer;
    FImgType: Integer;
    FImgSize: Integer;
    FBinMode: Integer;
    FMaxHeight: Integer;
    FMaxWidth: Integer;
    FNumOfControls: Integer;
    FROIHeight: Integer;
    FROIWidth: Integer;
    FStartX: Integer;
    FStartY: Integer;
    FFlip: Integer;
    FOldCamStatus: TCamStatus;
    FOldCamModel: AnsiString;
    FOldGain: Integer;
    FOldExposure: Integer;
    FOldExposureRange: Integer;
    FOldGamma: Integer;
    FOldUSBTraffic: Integer;
    FOldImgSize: Integer;
    FOldImgType: Integer;
    FOldFlip: Integer;
    FFrameGrabbed: Boolean;
  public
    constructor Create(CreateSuspended: Boolean);
    destructor Destroy; override;
    procedure Execute; override;
    procedure ClosePreview;
    procedure Initialize;
    procedure GetSettings;
    procedure SetPreviewSize;
    procedure SetSettings;
    procedure UpdatePreview;
    property Owner: TfrmASICam read FOwner write FOwner;
    property FrameGrabbed: Boolean read FFrameGrabbed write FFrameGrabbed;
    property FrameCount: Integer read FFrameCount write FFrameCount;
  end;

  TfrmASICam = class(TForm)
    pgPages: TPageControl;
    tabPreview: TTabSheet;
    tabCapture: TTabSheet;
    mnuMain: TMainMenu;
    menFile: TMenuItem;
    miExit: TMenuItem;
    menOptions: TMenuItem;
    miHelp: TMenuItem;
    tmrUpdate: TTimer;
    OpenPictureDialog: TOpenPictureDialog;
    tabStack: TTabSheet;
    pnlPreview: TPanel;
    pnlPreviewControls: TPanel;
    phPreview: TPhoto;
    pnlStartStop: TPanel;
    btnOpenCamera: TButton;
    btnCloseCamera: TButton;
    pnlHardware: TPanel;
    btnStartPreview: TButton;
    lblImageSize: TLabel;
    cmbImageSize: TComboBox;
    lblExposureRange: TLabel;
    cmbExposureRange: TComboBox;
    tbExposure: TTrackBar;
    lblExposure: TLabel;
    lblGain: TLabel;
    tbGain: TTrackBar;
    lblGamma: TLabel;
    tbGamma: TTrackBar;
    lblUSBTraffic: TLabel;
    tbUSBTraffic: TTrackBar;
    pnlCaptureControls: TPanel;
    pnlCapture: TPanel;
    phCapture: TPhoto;
    pnlStackControls: TPanel;
    pnlStack: TPanel;
    phStack: TPhoto;
    chkFlipY: TCheckBox;
    chkFlipX: TCheckBox;
    grpVideoFormat: TGroupBox;
    rbRAW8: TRadioButton;
    rbRGB24: TRadioButton;
    rbRAW16: TRadioButton;
    rbY8: TRadioButton;
    chkDespeckle: TCheckBox;
    btnAddSERFrame: TButton;
    btnShowSERFrame: TButton;
    btnSaveSERFile: TButton;
    lblAvg: TLabel;
    lblFrames: TLabel;
    lblElapsedTime: TLabel;
    edTicks: TEdit;
    edAverageFrames: TEdit;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure miExitClick(Sender: TObject);
    procedure btnOpenCameraClick(Sender: TObject);
    procedure btnCloseCameraClick(Sender: TObject);
    procedure btnStartPreviewClick(Sender: TObject);
    procedure tmrUpdateTimer(Sender: TObject);
    procedure pgPagesChange(Sender: TObject);
    procedure tbGainChange(Sender: TObject);
    procedure cmbExposureRangeChange(Sender: TObject);
    procedure cmbImageSizeChange(Sender: TObject);
    procedure tbGammaChange(Sender: TObject);
    procedure tbUSBTrafficChange(Sender: TObject);
    procedure tbExposureChange(Sender: TObject);
    procedure phPreviewResize(Sender: TObject);
    procedure pnlPreviewResize(Sender: TObject);
    procedure chkFlipXClick(Sender: TObject);
    procedure chkFlipYClick(Sender: TObject);
    procedure rbRAW8Click(Sender: TObject);
    procedure btnAddSERFrameClick(Sender: TObject);
    procedure btnShowSERFrameClick(Sender: TObject);
    procedure btnSaveSERFileClick(Sender: TObject);
  private
    { Private declarations }
    CamStatus: TCamStatus;
    Ini: TIniFile;
    ASICamDir: string;
    ImgTypes: Integer;
    ImgType: Integer;
    // Camera parameters
    CamModel: AnsiString;
    // Control parameters
    ImgSize: Integer;
    ImgWidth: Integer;
    ImgHeight: Integer;
    Gain: Integer;
    Exposure: Integer;
    ExposureRange: Integer;
    Gamma: Integer;
    USBTraffic: Integer;
    Flip: Integer;
    FrameCount: Integer;
    OldFrameCount: Integer;
  protected
  public
    CaptureThread: TCaptureThread;
    { Public declarations }
    procedure OpenCamera;
    procedure CloseCamera;
    procedure StartPreview;
    procedure StopCapture;
    procedure ToFront;
    procedure UpdateControls;
  end;

var
  frmASICam: TfrmASICam;
  T: Cardinal;

implementation

{$R *.dfm}
// =============================================================================

function GetDocPath: string;
// Get path to My Documents (no trailing backslash)
var
  FilePath: array [0 .. MAX_PATH] of Char;
  IDL: PItemIDList;
begin
  Result := '';
  if Succeeded(SHGetSpecialFolderLocation(0, CSIDL_PERSONAL, IDL)) then
    if SHGetPathFromIDList(IDL, FilePath) then
      Result := FilePath;
end;

// =============================================================================

// =============================================================================
//
// TCaptureThread
//
// =============================================================================

constructor TCaptureThread.Create(CreateSuspended: Boolean);
begin
  inherited Create(CreateSuspended);
end;

// =============================================================================

destructor TCaptureThread.Destroy;
begin
  inherited;
end;

// =============================================================================

procedure TCaptureThread.Execute;
var
  I: Integer;
  F: TFloatColor;
  DFC: Integer;
  OldDFC: Integer;
  Status: Integer;
  Col: TRGB4;
begin
  Initialize;
  DFC := 0;
  try
    while not Terminated do
    begin
      Synchronize(GetSettings);
      // Check if ONE camera is connected
      // This program supports a single camera only
      if (ASIGetNumOfConnectedCameras <> 1) then
        FCamStatus := csNoCamera;
      // State Machine
      if (FCamStatus <> FOldCamStatus) then
      begin
        // Mode has changed
        case FCamStatus of
          csNoCamera:
            begin
              // No camera; kill any capture window
              Synchronize(ClosePreview);
              FOldCamStatus := csNoCamera;
            end;
          csClosed:
            begin
              ASIStopVideoCapture(FCameraID);
              Sleep(1000);
              ASICloseCamera(FCameraID);
              FOldGain := -1;
              FOldExposure := -1;
              FOldExposureRange := -1;
              FOldGamma := -1;
              FOldUSBTraffic := -1;
              FOldImgSize := -1;
              FOldImgType := -1;
              FOldFlip := -1;
              Synchronize(ClosePreview);
              FOldCamStatus := csClosed;
            end;
          csOpened:
            begin
              if (FOldCamStatus = csCapturing) then
              begin
                // End capture
                ASIStopVideoCapture(FCameraID);
                Sleep(1000);
                // Close camera
                ASICloseCamera(FCameraID);
                Synchronize(ClosePreview);
              end;
              // Get camera info
              ASIGetCameraProperty(@FCamInfo, 0);
              // Get camera ID
              FCameraID := FCamInfo.CameraID;
              // Get camera model (e.g. ASI120MM)
              FMaxHeight := FCamInfo.MaxHeight;
              FMaxWidth := FCamInfo.MaxWidth;
              FCamModel := AnsiString(FCamInfo.Name);
              // Get supported video formats
              I := 0;
              FImgTypes := 0;
              while (FCamInfo.SupportedVideoFormat[I] <> ASI_IMG_END) do
              begin
                case FCamInfo.SupportedVideoFormat[I] of
                  ASI_IMG_RAW8:
                    FImgTypes := FImgTypes or IMG_TYPE_RAW8;
                  ASI_IMG_RGB24:
                    FImgTypes := FImgTypes or IMG_TYPE_RGB24;
                  ASI_IMG_RAW16:
                    FImgTypes := FImgTypes or IMG_TYPE_RAW16;
                  ASI_IMG_Y8:
                    FImgTypes := FImgTypes or IMG_TYPE_Y8;
                end;
                Inc(I);
              end;
              // Open camera
              ASIOpenCamera(FCameraID);
              // Initialize camera
              ASIInitCamera(FCameraID);
              // Get number of controls
              ASIGetNumOfControls(FCameraID, FNumOfControls);
              // Get control capabilities
              for I := 0 to FNumOfControls - 1 do
                ASIGetControlCaps(FCameraID, I, @FControlCaps[I]);
              FOldCamStatus := csOpened;
            end;
          csCapturing:
            begin
              // Create named window
              // Update capture parameters from ImgSize
              case FImgSize of
                1:
                  begin
                    FROIWidth := 1280;
                    FROIHeight := 960;
                    FBinMode := 1;
                  end;
                2:
                  begin
                    FROIWidth := 960;
                    FROIHeight := 960;
                    FBinMode := 1;
                  end;
                3:
                  begin
                    FROIWidth := 640;
                    FROIHeight := 480;
                    FBinMode := 1;
                  end;
                4:
                  begin
                    FROIWidth := 480;
                    FROIHeight := 480;
                    FBinMode := 1;
                  end;
                5:
                  begin
                    FROIWidth := 640;
                    FROIHeight := 480;
                    FBinMode := 2;
                  end;
              else
                begin
                  FROIWidth := FMaxWidth;
                  FROIHeight := FMaxHeight;
                  FBinMode := 1;
                end;
              end;
              if (FBinMode = 1) then
              begin
                FStartX := (FMaxWidth - FROIWidth) div 2;
                FStartY := (FMaxHeight - FROIHeight) div 2;
              end
              else if (FBinMode = 2) then
              begin
                FStartX := (FMaxWidth - 2 * FROIWidth) div 2;
                FStartY := (FMaxHeight - 2 * FROIHeight) div 2;
              end;
              // Set capture frame size
              FreeMem(FCapturedFrame);
              case FImgType of
                ASI_IMG_Y8, ASI_IMG_RAW8:
                  begin
                    FCapturedFrameSize := FROIWidth * FROIHeight;
                  end;
                ASI_IMG_RGB24:
                  begin
                    FCapturedFrameSize := FROIWidth * FROIHeight * 3;
                  end;
                ASI_IMG_RAW16:
                  begin
                    FCapturedFrameSize := FROIWidth * FROIHeight * 2;
                  end;
              end;
              GetMem(FCapturedFrame, FCapturedFrameSize);
              // Set ROI format
              ASISetROIFormat(FCameraID, FROIWidth, FROIHeight, FBinMode,
                FImgType);
              // Start image capture
              ASIStartVideoCapture(FCameraID);
              // Adjust preview window
              Synchronize(SetPreviewSize);
              FOldCamStatus := csCapturing;
            end;
        end;
      end;
      // Update control values
      if (FCamStatus = csOpened) or (FCamStatus = csCapturing) then
      begin
        // Gain
        if (FGain <> FOldGain) then
        begin
          ASISetControlValue(FCameraID, ASI_GAIN, FGain, ASI_FALSE);
          FOldGain := FGain;
        end;
        // Exposure
        if (FExposure <> FOldExposure) or (FExposureRange <> FOldExposureRange)
        then
        begin
          ASISetControlValue(FCameraID, ASI_EXPOSURE,
            FExposure * FExposureRange, ASI_FALSE);
          FOldExposure := FExposure;
          FOldExposureRange := FExposureRange;
        end;
        // Gamma
        if (FGamma <> FOldGamma) then
        begin
          ASISetControlValue(FCameraID, ASI_GAMMA, FGamma, ASI_FALSE);
          FOldGamma := FGamma;
        end;
        // USBTraffic
        if (FUSBTraffic <> FOldUSBTraffic) then
        begin
          ASISetControlValue(FCameraID, ASI_BANDWIDTHOVERLOAD, FUSBTraffic,
            ASI_FALSE);
          FOldUSBTraffic := FUSBTraffic;
        end;
        // Flip
        if (FFlip <> FOldFlip) then
        begin
          ASISetControlValue(FCameraID, ASI_FLIP, FFlip, ASI_FALSE);
          FOldFlip := FFlip;
        end;
      end;
      if (FCamStatus = csCapturing) then
      begin
        if (FImgSize <> FOldImgSize) or (FImgType <> FOldImgType) then
        begin
          // End capture
          ASIStopVideoCapture(FCameraID);
          // Flag image parameters to be updated on the next iteration
          FOldImgSize := FImgSize;
          FOldImgType := FImgType;
          Synchronize(ClosePreview);
          FOldCamStatus := csUnknown;
        end
        else
        begin
          Status := ASIGetVideoData(FCameraID, FCapturedFrame,
            FCapturedFrameSize, 100);
          if (Status = ASI_SUCCESS) then
          begin
            OldDFC := DFC;
            ASIGetDroppedFrames(FCameraID, DFC);
            if (OldDFC = DFC) then
            begin
              Inc(FFrameCount);
              case FImgType of
                ASI_IMG_Y8, ASI_IMG_RAW8:
                  begin
                    if (FActivePage = TAB_PREVIEW) then
                    begin
                      for I := 0 to FCapturedFrameSize - 1 do
                      begin
                        Col.rgbRed := PByte(Integer(FCapturedFrame) + I)^;
                        Col.rgbGreen := Col.rgbRed;
                        Col.rgbBlue := Col.rgbRed;
                        Col.rgbReserved := 0;
                        PRGB4(Integer(FOwner.phPreview.Layers[0].Bits) +
                          (I * SizeOf(TRGB4)))^ := Col;
                      end;
                    end
                    else if (FActivePage = TAB_CAPTURE) and (not FFrameGrabbed)
                    then
                    begin
                      for I := 0 to FCapturedFrameSize - 1 do
                      begin
                        F.fRed := PByte(Integer(FCapturedFrame) + I)^ shl 4;
                        PFloatColor
                          (Integer(FOwner.phPreview.Layers[0].FloatColors) +
                          (I * SizeOf(TFloatColor)))^ := F;
                      end;
                      FFrameGrabbed := True;
                    end;
                  end;
                ASI_IMG_RGB24:
                  begin
                    if (FActivePage = TAB_PREVIEW) then
                    begin
                      RGB3To4(FOwner.phPreview.Layers[0].Bits, FCapturedFrame,
                        FCapturedFrameSize div 3);
                    end
                    else if (FActivePage = TAB_CAPTURE) and (not FFrameGrabbed)
                    then
                    begin
                      for I := 0 to FCapturedFrameSize div 3 - 1 do
                      begin
                        F.fRed := PByte(Integer(FCapturedFrame) + I * 3)^ shl 4;
                        F.fGreen :=
                          PByte(Integer(FCapturedFrame) + I * 3 + 1)^ shl 4;
                        F.fBlue :=
                          PByte(Integer(FCapturedFrame) + I * 3 + 2)^ shl 4;
                        PFloatColor
                          (Integer(FOwner.phPreview.Layers[0].FloatColors) +
                          (I * SizeOf(TFloatColor)))^ := F;
                      end;
                      FFrameGrabbed := True;
                    end;
                  end;
                ASI_IMG_RAW16:
                  begin
                    if (FActivePage = TAB_PREVIEW) then
                    begin
                      for I := 0 to FCapturedFrameSize div 2 - 1 do
                      begin
                        Col.rgbRed :=
                          PWord(Integer(FCapturedFrame) + I * 2)^ shr 8;
                        Col.rgbGreen := Col.rgbRed;
                        Col.rgbBlue := Col.rgbRed;
                        Col.rgbReserved := 0;
                        PRGB4(Integer(FOwner.phPreview.Layers[0].Bits) +
                          (I * SizeOf(TRGB4)))^ := Col;
                      end;
                    end
                    else if (FActivePage = TAB_CAPTURE) and (not FFrameGrabbed)
                    then
                    begin
                      for I := 0 to FCapturedFrameSize div 2 - 1 do
                      begin
                        F.fRed := PWord(Integer(FCapturedFrame) + I * 2)^ shr 4;
                        PFloatColor
                          (Integer(FOwner.phPreview.Layers[0].FloatColors) +
                          (I * SizeOf(TFloatColor)))^ := F;
                      end;
                    end;
                    FFrameGrabbed := True;
                  end;
              end;
              Synchronize(UpdatePreview);
            end;
          end
          else
          begin
            // Get dropped frames to release hardware
            ASIGetDroppedFrames(FCameraID, DFC);
          end;
        end;
      end;
      // Update settings
      Synchronize(SetSettings);
      // Don't hog the CPU!
      Sleep(10);
    end;
  finally
    // Clean up on thread exit
    Synchronize(ClosePreview);
    FreeMem(FCapturedFrame);
  end;
end;

// =============================================================================

procedure TCaptureThread.GetSettings;
begin
  with FOwner do
  begin
    FActivePage := pgPages.ActivePageIndex;
    FCamStatus := CamStatus;
    FGain := Gain;
    FExposure := Exposure;
    FExposureRange := ExposureRange;
    FGamma := Gamma;
    FUSBTraffic := USBTraffic;
    FImgType := ImgType;
    FImgSize := ImgSize;
    FFlip := Flip;
  end;
end;

// =============================================================================

procedure TCaptureThread.Initialize;
begin
  ZeroMemory(@FControlCaps[0], SizeOf(TASI_CONTROL_CAPS) * MAX_CAM_CONTROLS);
  ZeroMemory(@FCamInfo, SizeOf(TASI_CAMERA_INFO));
  FCamStatus := csUnknown;
  FOldCamStatus := csUnknown;
  FOldCamModel := '';
  FGain := 50;
  FExposure := 1;
  FExposureRange := 10000;
  FGamma := 50;
  FUSBTraffic := 40;
  FImgSize := 0;
  FImgType := ASI_IMG_RAW8;
  FFlip := 0;
  FOldGain := -1;
  FOldExposure := -1;
  FOldExposureRange := -1;
  FOldGamma := -1;
  FOldUSBTraffic := -1;
  FOldImgSize := -1;
  FOldImgType := -1;
  FOldFlip := -1;
end;

// =============================================================================

procedure TCaptureThread.SetSettings;
begin
  with FOwner do
  begin
    // CamStatus := FCamStatus;
    CamModel := FCamModel;
    ImgTypes := FImgTypes;
    // Enable image type menuitems
    rbRAW8.Enabled := ((ImgTypes and IMG_TYPE_RAW8) <> 0);
    rbRGB24.Enabled := ((ImgTypes and IMG_TYPE_RGB24) <> 0);
    rbRAW16.Enabled := ((ImgTypes and IMG_TYPE_RAW8) <> 0);
    rbY8.Enabled := ((ImgTypes and IMG_TYPE_Y8) <> 0);
  end;
end;

// =============================================================================

procedure TCaptureThread.SetPreviewSize;
begin
  // Resize preview window bitmap
  with FOwner do
  begin
    ImgWidth := 0;
    phPreview.Layers[0].Width := FROIWidth;
    phPreview.Layers[0].Height := FROIHeight;
    phPreview.Width := Round((phPreview.Height * FROIWidth) / FROIHeight);
    phCapture.Layers[0].Height := phPreview.Layers[0].Height;
    phCapture.Layers[0].Width := phPreview.Layers[0].Width;
    phStack.Layers[0].Height := phPreview.Layers[0].Height;
    phStack.Layers[0].Width := phPreview.Layers[0].Width;
    case FImgType of
      ASI_IMG_Y8, ASI_IMG_RAW8:
        begin
          phPreview.Layers[0].Monochrome := True;
          phCapture.Layers[0].Monochrome := True;
          phStack.Layers[0].Monochrome := True;
        end;
      ASI_IMG_RGB24:
        begin
          phPreview.Layers[0].Monochrome := False;
          phCapture.Layers[0].Monochrome := False;
          phStack.Layers[0].Monochrome := False;
        end;
      ASI_IMG_RAW16:
        begin
          phPreview.Layers[0].Monochrome := True;
          phCapture.Layers[0].Monochrome := True;
          phStack.Layers[0].Monochrome := True;
        end;
    end;
    phPreview.Paint;
    phCapture.Paint;
    phStack.Paint;
    ImgWidth := FROIWidth;
    ImgHeight := FROIHeight;
  end;
end;

// =============================================================================

procedure TCaptureThread.UpdatePreview;
begin
  with FOwner do
  begin
    // Update framecount
    FrameCount := FFrameCount;
    // Update preview window
    if (FActivePage = TAB_PREVIEW) then
      phPreview.Paint;
  end;
end;

// =============================================================================

procedure TCaptureThread.ClosePreview;
begin
  // Set preview window to black
  FOwner.phPreview.Layers[0].Fill(clBlack);
  FOwner.phPreview.Paint;
end;

// =============================================================================
//
// TfrmASICam
//
// =============================================================================


// =============================================================================

procedure TfrmASICam.CloseCamera;
begin
  CamStatus := csClosed;
  pnlHardware.Hide;
  Caption := 'ASICam';
end;

// =============================================================================

procedure TfrmASICam.OpenCamera;
begin
  CamStatus := csOpened;
  pnlHardware.Visible := (CamStatus = csOpened);
end;

// =============================================================================

procedure TfrmASICam.StartPreview;
begin
  CamStatus := csCapturing;
end;

// =============================================================================

procedure TfrmASICam.StopCapture;
begin
  CamStatus := csOpened;
end;

// =============================================================================

procedure TfrmASICam.ToFront;
begin
  // Bring first instance to front
  Application.Restore;
  Application.BringToFront;
end;

// =============================================================================

// Update controls from variables
procedure TfrmASICam.UpdateControls;
begin
  chkFlipX.Checked := ((Flip and 1) <> 0);
  chkFlipY.Checked := ((Flip and 2) <> 0);
  rbRAW8.Enabled := ((ImgTypes and IMG_TYPE_RAW8) <> 0);
  rbRGB24.Enabled := ((ImgTypes and IMG_TYPE_RGB24) <> 0);;
  rbRAW16.Enabled := ((ImgTypes and IMG_TYPE_RAW8) <> 0);
  rbY8.Enabled := ((ImgTypes and IMG_TYPE_Y8) <> 0);
  rbRAW8.Checked := (ImgType = ASI_IMG_RAW8);
  rbRGB24.Checked := (ImgType = ASI_IMG_RGB24);
  rbRAW16.Checked := (ImgType = ASI_IMG_RAW16);
  rbY8.Checked := (ImgType = ASI_IMG_Y8);
  cmbImageSize.ItemIndex := ImgSize;
  case ExposureRange of
    100:
      cmbExposureRange.ItemIndex := 0;
    1000:
      cmbExposureRange.ItemIndex := 1;
    10000:
      cmbExposureRange.ItemIndex := 2;
    100000:
      cmbExposureRange.ItemIndex := 3;
    1000000:
      cmbExposureRange.ItemIndex := 4;
    10000000:
      cmbExposureRange.ItemIndex := 5;
    100000000:
      cmbExposureRange.ItemIndex := 6;
  end;
  tbExposure.Position := Exposure;
  tbGain.Position := Gain;
  tbGamma.Position := Gamma;
  tbUSBTraffic.Position := USBTraffic;
end;

// =============================================================================

procedure TfrmASICam.FormCreate(Sender: TObject);
begin
  // Prevent multiple instances
  // Instances.TriggerProc := ToFront;
  // Creat ASICam folder in My Documents
  ASICamDir := GetDocPath + '\ASICam';
  if not DirectoryExists(ASICamDir) then
  begin
    ForceDirectories(ASICamDir);
    ForceDirectories(ASICamDir + '\SERFiles');
  end;
  // Create Ini file
  Ini := TIniFile.Create(ASICamDir + '\ASICam.ini');
  // Get preferences from ini
  Width := Ini.ReadInteger('Preferences', 'Width', 830);
  Height := Ini.ReadInteger('Preferences', 'Height', 600);
  Left := Ini.ReadInteger('Preferences', 'Left', (Screen.Width - Width) div 2);
  Top := Ini.ReadInteger('Preferences', 'Top', (Screen.Height - Height) div 2);
  // Get controls from ini
  Gain := Ini.ReadInteger('Controls', 'Gain', 50);
  ExposureRange := Ini.ReadInteger('Controls', 'ExposureRange', 10000);
  Exposure := Ini.ReadInteger('Controls', 'Exposure', 1);
  Gamma := Ini.ReadInteger('Controls', 'Gamma', 50);
  USBTraffic := Ini.ReadInteger('Controls', 'USBTraffic', 40);
  Flip := Ini.ReadInteger('Controls', 'Flip', 0);
  chkFlipX.Checked := ((Flip and 1) <> 0);
  chkFlipY.Checked := ((Flip and 2) <> 0);
  ImgType := Ini.ReadInteger('Controls', 'ImgType', ASI_IMG_RAW8);
  ImgSize := Ini.ReadInteger('Controls', 'ImgSize', 0);
  lblGain.Caption := 'Gain - ' + IntToStr(Gain);
  lblGamma.Caption := 'Gamma - ' + IntToStr(Gamma);
  lblUSBTraffic.Caption := 'USB Traffic - ' + IntToStr(USBTraffic);
  UpdateControls;
  tbExposureChange(Self);
  // Hide the hardware panel
  pnlHardware.Visible := False;
  // Create capture thread
  CaptureThread := TCaptureThread.Create(True);
  CaptureThread.FreeOnTerminate := False;
  CaptureThread.Owner := Self;
  Show;
  Refresh;
  phPreview.LayerCount := 1;
  phPreview.Layers[0].PixelDepth := 12;
  phPreview.Paint;
  phCapture.Layers[0].PixelDepth := 12;
  phStack.Layers[0].PixelDepth := 12;
  CaptureThread.Start;
  tmrUpdate.Enabled := True;
end;

// =============================================================================

procedure TfrmASICam.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  // Save preferences
  Ini.WriteInteger('Preferences', 'Width', Width);
  Ini.WriteInteger('Preferences', 'Height', Height);
  Ini.WriteInteger('Preferences', 'Left', Left);
  Ini.WriteInteger('Preferences', 'Top', Top);
  // Save controls
  Ini.WriteInteger('Controls', 'ExposureRange', ExposureRange);
  Ini.WriteInteger('Controls', 'Exposure', Exposure);
  Ini.WriteInteger('Controls', 'Gain', Gain);
  Ini.WriteInteger('Controls', 'Gamma', Gamma);
  Ini.WriteInteger('Controls', 'USBTraffic', USBTraffic);
  Ini.WriteInteger('Controls', 'Flip', Flip);
  Ini.WriteInteger('Controls', 'ImgType', ImgType);
  Ini.WriteInteger('Controls', 'ImgSize', ImgSize);
  // Clean up
  Ini.Free;
  // Kill CaptureThread
  CaptureThread.Terminate;
  CaptureThread.WaitFor;
  CaptureThread.Free;
end;

// =============================================================================

// Close program
procedure TfrmASICam.miExitClick(Sender: TObject);
begin
  Close;
end;

// =============================================================================

// Open button

procedure TfrmASICam.btnOpenCameraClick(Sender: TObject);
begin
  OpenCamera;
end;

// =============================================================================

procedure TfrmASICam.pgPagesChange(Sender: TObject);
begin
  pnlStartStop.Visible := (pgPages.ActivePage = tabPreview);
  pnlHardware.Visible := (pgPages.ActivePage = tabPreview) and
    (CamStatus <> csClosed);
end;

// =============================================================================

procedure TfrmASICam.phPreviewResize(Sender: TObject);
begin
  if (ImgWidth > 0) and (ImgHeight > 0) then
    phPreview.Width := Round((phPreview.Height * ImgWidth) / ImgHeight);
  phPreview.Left := (pnlPreview.Width - phPreview.Width) div 2;
  phCapture.Width := phPreview.Width;
  phCapture.Left := phPreview.Left;
  phStack.Width := phPreview.Width;
  phStack.Left := phPreview.Left;
end;

// =============================================================================

procedure TfrmASICam.pnlPreviewResize(Sender: TObject);
begin
  phPreview.Height := pnlPreview.Height - 8;
  phPreview.Left := (pnlPreview.Width - phPreview.Width) div 2;
  phCapture.Height := phPreview.Height;
  phCapture.Left := phPreview.Left;
  phStack.Height := phPreview.Height;
  phStack.Left := phPreview.Left;
end;

// =============================================================================

procedure TfrmASICam.rbRAW8Click(Sender: TObject);
begin
  ImgType := ASI_IMG_RAW8;
  if (rbRGB24.Enabled and rbRGB24.Checked) then
    ImgType := ASI_IMG_RGB24
  else if (rbRAW16.Enabled and rbRAW16.Checked) then
    ImgType := ASI_IMG_RAW16
  else if (rbY8.Enabled and rbY8.Checked) then
    ImgType := ASI_IMG_Y8;
end;

// =============================================================================

procedure TfrmASICam.btnCloseCameraClick(Sender: TObject);
begin
  CloseCamera;
end;

// =============================================================================

// Status timer
procedure TfrmASICam.tmrUpdateTimer(Sender: TObject);
var
  I: Integer;
  OldCol, NewCol: TFloatColor;
  Src, Dst: PFloatColor;
  Mult, Rmdr: Single;
begin
  T := TimeGetTime;
  if (pgPages.ActivePage = tabCapture) then
  begin
    if (CaptureThread.FrameGrabbed) and (FrameCount <> OldFrameCount) and
      (phPreview.Layers[0].Width = phCapture.Layers[0].Width) and
      (phPreview.Layers[0].Height = phCapture.Layers[0].Height) then
    begin
      T := TimeGetTime;
      if (StrToInt(edAverageFrames.Text) <= 1) then
        CopyMemory(phCapture.Layers[0].FloatColors,
          phPreview.Layers[0].FloatColors, phPreview.Layers[0].SizeF)
      else
      begin
        Mult := 1 / StrToInt(edAverageFrames.Text);
        Rmdr := 1 - Mult;
        Src := phPreview.Layers[0].FloatColors;
        Dst := phCapture.Layers[0].FloatColors;
        for I := 0 to phPreview.Layers[0].PixelCount - 1 do
        begin
          OldCol := Dst^;
          NewCol := Src^;
          OldCol.fRed := OldCol.fRed * Rmdr;
          if phCapture.Layers[0].Monochrome then
          begin
            OldCol.fGreen := OldCol.fRed;
            OldCol.fBlue := OldCol.fRed;
          end
          else
          begin
            OldCol.fGreen := OldCol.fGreen * Rmdr;
            OldCol.fBlue := OldCol.fBlue * Rmdr;
          end;
          NewCol.fRed := NewCol.fRed * Mult + OldCol.fRed;
          if phCapture.Layers[0].Monochrome then
          begin
            NewCol.fGreen := NewCol.fRed;
            NewCol.fBlue := NewCol.fRed;
          end
          else
          begin
            NewCol.fGreen := NewCol.fGreen * Mult + OldCol.fGreen;
            NewCol.fBlue := NewCol.fBlue * Mult + OldCol.fBlue;
          end;
          Dst^ := NewCol;
          Inc(Src);
          Inc(Dst);
        end;
      end;
      if (chkDespeckle.Checked) then
        phCapture.Layers[0].DespeckleF;
      // Update capture window
      phCapture.Layers[0].ShowFloatColors(phCapture.Layers[0], 1 / 16);
      phCapture.Paint;
      OldFrameCount := FrameCount;
      // Prime next grab
      CaptureThread.FrameGrabbed := False;
      edTicks.Text := IntToStr(TimeGetTime - T);
    end;
  end;
end;

// =============================================================================

// Exposure change slider
procedure TfrmASICam.tbExposureChange(Sender: TObject);
begin
  if (ExposureRange <= 0) or (cmbExposureRange.ItemIndex < 0) then
  begin
    ExposureRange := 10000;
    cmbExposureRange.ItemIndex := 2;
  end;
  Exposure := tbExposure.Position;
  case cmbExposureRange.ItemIndex of
    0:
      lblExposure.Caption := 'Exposure - ' +
        IntToStr(tbExposure.Position * 100) + 's';
    1:
      lblExposure.Caption := 'Exposure - ' +
        IntToStr(tbExposure.Position) + 'ms';
    2:
      lblExposure.Caption := 'Exposure - ' +
        IntToStr(tbExposure.Position * 10) + 'ms';
    3:
      lblExposure.Caption := 'Exposure - ' +
        IntToStr(tbExposure.Position * 100) + 'ms';
    4:
      lblExposure.Caption := 'Exposure - ' +
        IntToStr(tbExposure.Position) + 's';
    5:
      lblExposure.Caption := 'Exposure - ' +
        IntToStr(tbExposure.Position * 10) + 's';
    6:
      lblExposure.Caption := 'Exposure - ' +
        IntToStr(tbExposure.Position * 100) + 's';
  end;
end;

// =============================================================================

// Gain change slider
procedure TfrmASICam.tbGainChange(Sender: TObject);
begin
  Gain := tbGain.Position;
  lblGain.Caption := 'Gain - ' + IntToStr(Gain);
end;

// =============================================================================

// Gamma change slider
procedure TfrmASICam.tbGammaChange(Sender: TObject);
begin
  Gamma := tbGamma.Position;
  lblGamma.Caption := 'Gamma - ' + IntToStr(Gamma);
end;

// =============================================================================

// USB Traffic change slider
procedure TfrmASICam.tbUSBTrafficChange(Sender: TObject);
begin
  USBTraffic := tbUSBTraffic.Position;
  lblUSBTraffic.Caption := 'USB Traffic - ' + IntToStr(USBTraffic);
end;

// =============================================================================

procedure TfrmASICam.cmbExposureRangeChange(Sender: TObject);
begin
  ExposureRange := Round(Power(10, cmbExposureRange.ItemIndex) * 100);
  // Set Exposure to min for that range
  Exposure := 1;
  tbExposure.Position := 1;
  tbExposureChange(Self);
end;

// =============================================================================

// Image size changed
procedure TfrmASICam.cmbImageSizeChange(Sender: TObject);
begin
  // Set image size
  ImgSize := cmbImageSize.ItemIndex;
end;

// =============================================================================

// Start Capture button
procedure TfrmASICam.btnSaveSERFileClick(Sender: TObject);
begin
  phCapture.Layers[0].SaveSERFile(GetDocPath + '\ASICam\SERFiles\test.ser');
end;

// =============================================================================

procedure TfrmASICam.btnAddSERFrameClick(Sender: TObject);
begin
  phCapture.Layers[0].AddSERFrame;
end;

// =============================================================================

// Stop Capture button
procedure TfrmASICam.btnShowSERFrameClick(Sender: TObject);
begin
  phCapture.Layers[0].GetSERFrame(0, phStack.Layers[0]);
  phStack.Layers[0].ShowFloatColors(phStack.Layers[0], 1 / 16);
  phStack.Paint;
  pgPages.ActivePage := tabStack;
end;

// =============================================================================

procedure TfrmASICam.btnStartPreviewClick(Sender: TObject);
begin
  StartPreview;
end;

// =============================================================================

procedure TfrmASICam.chkFlipXClick(Sender: TObject);
begin
  if (chkFlipX.Checked) then
    Flip := Flip or 1
  else
    Flip := Flip and $FFFFFFFE;
end;

// =============================================================================

procedure TfrmASICam.chkFlipYClick(Sender: TObject);
begin
  if (chkFlipY.Checked) then
    Flip := Flip or 2
  else
    Flip := Flip and $FFFFFFFD;
end;

// =============================================================================

initialization

// Set MMSYSTEM granularity to 1ms
TimeBeginPeriod(1);

finalization

TimeEndPeriod(1);

end.
