// -----------------------------------------------------------------------------
// API for ZWO astro cameras
// Requires latest ASICamera2.dll from ASI Windows SDK
// (http://astronomy-imaging-camera.com/software -> SDK & Upgrade Tool)
// Requires ZWO Windows driver 3.0.0.2 or later
// (http://astronomy-imaging-camera.com/software -> Windows driver)
//
// Original translation by:
// Youngjae Ha (sbrngm@gmail.com)
// http://sbrngm.tistory.com
//
// Updated by:
// Richard Shotbolt
// http://www.shotbolt.com
//
// Last update : RS 2017-12-31
// Updated for latest ASICamera2.dll and Windows drivers ver 3.0.0.2
// -----------------------------------------------------------------------------

unit ASICamera2;

interface

// -----------------------------------------------------------------------------

// ASI Camera DLL
const
  ASICAMERA_API = 'ASICamera2.dll';

// ASI_BAYER_PATTERN
const
  ASI_BAYER_RG = 0;
  ASI_BAYER_BG = 1;
  ASI_BAYER_GR = 2;
  ASI_BAYER_GB = 3;

// ASI_IMG_TYPE
const
  ASI_IMG_END = -1; // stop enumerating when SupportedVideoFormat = ASI_IMG_END
  ASI_IMG_RAW8 = 0; // 8 bpp mono
  ASI_IMG_RGB24 = 1; // 8 bpp RGB color
  ASI_IMG_RAW16 = 2; // 16 bpp mono
  ASI_IMG_Y8 = 3; // 8 bpp momo (color cameras)

// ASI_GUIDE_DIRECTION
const
  ASI_GUIDE_NORTH = 0;
  ASI_GUIDE_SOUTH = 1;
  ASI_GUIDE_EAST = 2;
  ASI_GUIDE_WEST = 3;

// ASI_FLIP_STATUS
const
  ASI_FLIP_NONE = 0; // no flip
  ASI_FLIP_HORIZ = 1; // horizontal flip
  ASI_FLIP_VERT = 2; // vertical flip
  ASI_FLIP_BOTH = 3; // both horizontal and vertical flip

// ASI_EXPOSURE_STATUS
const
  ASI_EXP_IDLE = 0; // idle, ready to start exposure
  ASI_EXP_WORKING = 1; // exposure in progress
  ASI_EXP_SUCCESS = 2; // exposure OK, image can be read out
  ASI_EXP_FAILED = 3; // exposure failure, need to restart exposure

// ASI_ERROR_CODE
const
  ASI_SUCCESS = 0; // operation successful
  ASI_ERROR_INVALID_INDEX = 1; // no camera connected or invalid index value
  ASI_ERROR_INVALID_ID = 2; // invalid ID
  ASI_ERROR_INVALID_CONTROL_ID = 3; // invalid Control ID
  ASI_ERROR_CAMERA_CLOSED = 4; // camera isn't open
  ASI_ERROR_CAMERA_REMOVED = 5; // failed to find camera, maybe disconnected
  ASI_ERROR_INVALID_PATH = 6; // cannot find the path of the file
  ASI_ERROR_INVALID_FILEFORMAT = 7; // wrong video file format
  ASI_ERROR_INVALID_SIZE = 8; // wrong video format size
  ASI_ERROR_INVALID_IMGTYPE = 9; // unsupported image format
  ASI_ERROR_OUTOF_BOUNDARY = 10; // image out of bounds
  ASI_ERROR_TIMEOUT = 11; // timeout
  ASI_ERROR_INVALID_SEQUENCE = 12; // invalid sequence, stop capture first
  ASI_ERROR_BUFFER_TOO_SMALL = 13; // buffer size too small
  ASI_ERROR_VIDEO_MODE_ACTIVE = 14; // video mode is active
  ASI_ERROR_EXPOSURE_IN_PROGRESS = 15; // exposure in progress
  ASI_ERROR_GENERAL_ERROR = 16; // general error e.g. invalid value
  ASI_ERROR_END = 17;

// ASI_BOOL
const
  ASI_FALSE = 0; // false
  ASI_TRUE = 1; // true

// ASI_CONTROL_TYPE
const
  ASI_GAIN = 0; // gain
  ASI_EXPOSURE = 1; // exposure time, microsec
  ASI_GAMMA = 2; // gamma
  ASI_WB_R = 3; // white balance, red component
  ASI_WB_B = 4; // white balance, blue component
  ASI_BRIGHTNESS = 5; // pixel value offset (a bias, not a scale factor)
  ASI_BANDWIDTHOVERLOAD = 6; // total data transfer rate percentage
  ASI_OVERCLOCK = 7; // overclock
  ASI_TEMPERATURE = 8; // sensor temperature,10 * actual temperature
  ASI_FLIP = 9; // image flip mode
  ASI_AUTO_MAX_GAIN = 10; // max gain in auto mode
  ASI_AUTO_MAX_EXP = 11; // max exposure in auto mode (microsec)
  ASI_AUTO_MAX_BRIGHTNESS = 12; // target brightness in auto mode
  ASI_HARDWARE_BIN = 13; // hardware pixel binning
  ASI_HIGH_SPEED_MODE = 14; // high-speed mode
  ASI_COOLER_POWER_PERC = 15; // cooler power percentage
  ASI_TARGET_TEMP = 16; // sensor target temp, don't multiply by 10
  ASI_COOLER_ON = 17; // cooler on/off
  ASI_MONO_BIN = 18; // monochrome binning for color camera
  ASI_FAN_ON = 19; // cooler fan on/off
  ASI_PATTERN_ADJUST = 20; // currently only supported by 1600 mono camera
  ASI_ANTI_DEW_HEATER = 21; // anti-dewing heater on/off

// -----------------------------------------------------------------------------

// ASI_CAMERA_INFO
type
  PASI_CAMERA_INFO = ^TASI_CAMERA_INFO;

  TASI_CAMERA_INFO = record
    Name: array [0 .. 63] of AnsiChar; // camera model string
    CameraID: Integer; // camera ID, 0 = first camera found
    MaxHeight: Longint; // max height in pixels
    MaxWidth: Longint; // max width in pixels
    IsColorCam: Integer; // ASI_TRUE = color camera
    BayerPattern: Integer; // Bayer pattern
    SupportedBins: array [0 .. 15] of Integer; // supported binning modes
    SupportedVideoFormat: array [0 .. 7] of Integer; // supported video modes
    PixelSize: Double; // sensor chip pixel size in microns
    MechanicalShutter: Integer; // ASI_TRUE = mechanical shutter present
    ST4Port: Integer; // ASI_TRUE = ST4 guiding port present
    IsCoolerCam: Integer; // ASI_TRUE = camera has a cooler
    IsUSB3Host: Integer; // ASI_TRUE = camera using a USB3 port
    IsUSB3Camera: Integer; // ASI_TRUE = camera has a USB3 port
    ElecPerADU: Single; // electrons per A/D step
    OffsetLGain: Integer; // lowest supported gain
    OffsetHGain: Integer; // highest supported gain
    Unused: array [0 .. 15] of AnsiChar;
  end;

// ASI_CONTROL_CAPS
type
  PASI_CONTROL_CAPS = ^TASI_CONTROL_CAPS;

  TASI_CONTROL_CAPS = packed record
    Name: array [0 .. 63] of AnsiChar; // name string of this control
    Description: array [0 .. 127] of AnsiChar; // description of this control
    ControlID: Integer; // control ID used to get/set values
    MaxValue: Longint; // max value for this control
    MinValue: Longint; // min value for this control
    DefaultValue: Longint; // default value for this control
    IsAutoSupported: Integer; // ASI_TRUE = control has auto mode
    IsWritable: Integer; // ASI_TRUE = control can be changed manually
    ControlType: Integer; // ASI_CONTROL_TYPE for this control
    Unused: array [0 .. 31] of AnsiChar;
  end;

// ASI_ID
type
  PASI_ID = ^TASI_ID;

  // ID to be read/written from/to camera flash ROM, 8 bytes total
  TASI_ID = packed record
    ID: array [0 .. 7] of AnsiChar;
  end;

// ASI_PID
type
  PASI_PID = ^TASI_PID;

  // product IDs
  TASI_PID = packed record
    PID: array [0 .. 15] of Integer;
  end;

// -----------------------------------------------------------------------------
//
// ASICamera2.dll function declarations
//
// -----------------------------------------------------------------------------

// *****************************************************************************
// ASIGetNumOfConnectedCameras
// Description:
// Get number of connected ASI cameras
//
// Params:
// none
//
// Return:
// number of connected ASI cameras
// *****************************************************************************
function ASIGetNumOfConnectedCameras(): Integer; cdecl; external ASICAMERA_API;

// *****************************************************************************
// ASIGetCameraProperty
// Description:
// Get the properties of the selected camera
//
// Params:
// pASICameraInfo : pointer to TASICameraInfo structure
// iCameraIndex : camera index, 0 = first
//
// Return:
// ASI_SUCCESS
// ASI_ERROR_INVALID_INDEX : no camera connected or index of bounds
// *****************************************************************************
function ASIGetCameraProperty(pASICameraInfo: PASI_CAMERA_INFO;
  iCameraIndex: Integer): Integer; cdecl; external ASICAMERA_API;

// *****************************************************************************
// ASIOpenCamera
// Description:
// Open the selected camera, must be followed by ASIInitCamera if successful
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
//
// Return:
// ASI_SUCCESS
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// ASI_ERROR_CAMERA_REMOVED : failed to find camera, maybe disconnected
// *****************************************************************************
function ASIOpenCamera(iCameraID: Integer): Integer; cdecl;
  external ASICAMERA_API;

// *****************************************************************************
// ASIInitCamera
// Description:
// Initialize camera, must be preceded by ASIOpenCamera
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
//
// Return:
// ASI_SUCCESS
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// ASI_ERROR_CAMERA_REMOVED : failed to find camera, maybe disconnected
// *****************************************************************************
function ASIInitCamera(iCameraID: Integer): Integer; cdecl;
  external ASICAMERA_API;

// *****************************************************************************
// ASICloseCamera
// Description:
// Close camera and release resources
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
//
// Return:
// ASI_SUCCESS : returns success even if the camera is already closed
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// *****************************************************************************
function ASICloseCamera(iCameraID: Integer): Integer; cdecl;
  external ASICAMERA_API;

// *****************************************************************************
// ASIGetNumOfControls
// Description:
// Get number of supported controls
// Must be called after ASIOpenCamera and ASIInitCamera (not optional)
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
// piNumberOfControls : Integer var parameter receives number of controls
//
// Return:
// ASI_SUCCESS
// ASI_ERROR_CAMERA_CLOSED : camera isn't open
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// *****************************************************************************
function ASIGetNumOfControls(iCameraID: Integer;
  var piNumberOfControls: Integer): Integer; cdecl; external ASICAMERA_API;

// *****************************************************************************
// ASIGetControlCaps
// Description:
// Get properties of a given camera control
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
// iControlIndex : index of control
// pControlCaps : pointer to TASI_CONTROL_CAPS structure to receive settings
//
// Return:
// ASI_SUCCESS
// ASI_ERROR_CAMERA_CLOSED : camera isn't open
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// *****************************************************************************
function ASIGetControlCaps(iCameraID: Integer; iControlIndex: Integer;
  pControlCaps: PASI_CONTROL_CAPS): Integer; cdecl; external ASICAMERA_API;

// *****************************************************************************
// ASIGetControlValue
// Description:
// Get control property value and auto status
// Note: the value of the temperature is the float value * 10
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
// ControlType : control type (see ASI_CONTROL_TYPE)
// plValue : LongInt var parameter to receive the current value of the control
// pbAuto : Integer var parameter to receive the auto status of the control
//
// Return:
// ASI_SUCCESS
// ASI_ERROR_CAMERA_CLOSED : camera isn't open
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// ASI_ERROR_INVALID_CONTROL_ID : invalid ControlType
// *****************************************************************************
function ASIGetControlValue(iCameraID: Integer; ControlType: Integer;
  var plValue: Longint; var pbAuto: Integer): Integer; cdecl;
  external ASICAMERA_API;

// *****************************************************************************
// ASISetControlValue
// Description:
// Set control property value and auto status
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
// ControlType : control type (see ASI_CONTROL_TYPE)
// lValue : new value for the control
// bAuto : new value for the auto status of the control
//
// if bAuto is ASI_TRUE then lValue should be set to its current value
//
// Return:
// ASI_SUCCESS
// ASI_ERROR_CAMERA_CLOSED : camera isn't open
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// ASI_ERROR_INVALID_CONTROL_ID : invalid ControlType
// *****************************************************************************
function ASISetControlValue(iCameraID: Integer; ControlType: Integer;
  lValue: Longint; bAuto: Integer): Integer; cdecl; external ASICAMERA_API;

// *****************************************************************************
// ASIGetROIFormat
// Description:
// Get the ROI area
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
// piWidth : Integer var parameter; width of the ROI area
// piHeight : Integer var parameter; height of the ROI area
// piBin : Integer var parameter; binning method, bin1 = 1, bin2 = 2
// pImg_type : Integer var parameter; image format
//
// Return:
// ASI_SUCCESS
// ASI_ERROR_CAMERA_CLOSED : camera isn't open
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// *****************************************************************************
function ASIGetROIFormat(iCameraID: Integer; var piWidth: Integer;
  var piHeight: Integer; var piBin: Integer; var pImg_type: Integer): Integer;
  cdecl; external ASICAMERA_API;

// *****************************************************************************
// ASISetROIFormat
// Description:
// Set the ROI area before capture
// Stop capture before setting ROI format
// The width and height is the value after binning
// e.g. width = 640 and height = 480 for 640 X 480 @BIN2
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
// iWidth : width of the ROI area
// iHeight : height of the ROI area
// iBin : binning method, bin1 = 1, bin2 = 2
// Img_type : required image format
//
// Return:
// ASI_SUCCESS
// ASI_ERROR_CAMERA_CLOSED : camera isn't open
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// ASI_ERROR_INVALID_SIZE : unsupported video format size
// ASI_ERROR_INVALID_IMGTYPE : unsupported image format
// *****************************************************************************
function ASISetROIFormat(iCameraID: Integer; iWidth: Integer; iHeight: Integer;
  iBin: Integer; Img_type: Integer): Integer; cdecl; external ASICAMERA_API;

// *****************************************************************************
// ASIGetStartPos
// Description:
// Get the start position of the ROI area.
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
// piStartX : Integer var parameter; X coordinate of start position
// piStartY : Integer var parameter; Y coordinate of start position
//
// Return:
// ASI_SUCCESS
// ASI_ERROR_CAMERA_CLOSED : camera isn't open
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// *****************************************************************************
function ASIGetStartPos(iCameraID: Integer; var piStartX: Integer;
  var piStartY: Integer): Integer; cdecl; external ASICAMERA_API

// *****************************************************************************
// ASISetStartPos
// Description:
// Set the start position of the ROI area.
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
// iStartX : X coordinate of start position
// iStartY : Y coordinate of start position
//
// Return:
// ASI_SUCCESS
// ASI_ERROR_CAMERA_CLOSED : camera isn't open
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// ASI_ERROR_OUTOF_BOUNDARY : X/Y coordinate puts ROI out of bounds
// *****************************************************************************
  function ASISetStartPos(iCameraID: Integer; iStartX: Integer;
  iStartY: Integer): Integer; cdecl; external ASICAMERA_API

// *****************************************************************************
// ASIGetDroppedFrames
// Description:
// Get dropped frame count during video capture
// Dropped frames happen when USB traffic or disk write speed is slow
// Resets to 0 when capture is stopped
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
// piDropFrames : Integer var parameter, dropped frame count
//
// Return:
// ASI_SUCCESS
// ASI_ERROR_CAMERA_CLOSED : camera isn't open
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// *****************************************************************************
  function ASIGetDroppedFrames(iCameraID: Integer; var piDropFrames: Integer)
  : Integer; cdecl; external ASICAMERA_API;

// *****************************************************************************
// ASIEnableDarkSubtract
// Description:
// Provide path to a dark frame file and enable dark frame subtract
// This is used when there are hot pixels or need to do long exposure
// File can be made from the "dark subtract" function
// of the "video capture filter" directshow page
// The dark file's size should be the same of camera's max width and height
// and should be RGB8 raw format. It will work even you change the ROI setting
// It will only correct the hot pixels if output format is ASI_IMG_RAW16
// Setting is saved in the registry
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
// pcBMPPath : pointer to full dark frame file string
//
// return:
// ASI_SUCCESS
// ASI_ERROR_CAMERA_CLOSED : camera isn't open
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// ASI_ERROR_INVALID_PATH : cannot find the path of the file
// ASI_ERROR_INVALID_FILEFORMAT : dark frame file is invalid
// *****************************************************************************
function ASIEnableDarkSubtract(iCameraID: Integer; pcBMPPath: PAnsiChar)
  : Integer; cdecl; external ASICAMERA_API;

// *****************************************************************************
// ASIDisableDarkSubtract
// Description:
// Disable the dark subtract function
// Call it at start if you don't want to use it
// because dark subtract is saved in registry on windows platform
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
//
// Return:
// ASI_SUCCESS
// ASI_ERROR_CAMERA_CLOSED : camera isn't open
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// *****************************************************************************
function ASIDisableDarkSubtract(iCameraID: Integer): Integer; cdecl;
  external ASICAMERA_API;

// *****************************************************************************
// ASIStartVideoCapture
// Description:
// Start video capture
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
//
// Return:
// ASI_SUCCESS
// ASI_ERROR_CAMERA_CLOSED : camera isn't open
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// *****************************************************************************
function ASIStartVideoCapture(iCameraID: Integer): Integer; cdecl;
  external ASICAMERA_API;

// *****************************************************************************
// ASIStopVideoCapture
// Description:
// Stop video capture
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
//
// Return:
// ASI_SUCCESS
// ASI_ERROR_CAMERA_CLOSED : camera isn't open
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// *****************************************************************************
function ASIStopVideoCapture(iCameraID: Integer): Integer; cdecl;
  external ASICAMERA_API;

// *****************************************************************************
// ASIGetVideoData
// Description:
// Get frame from the camera
// Call as often as possible, otherwise frames will be discarded
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
// pBuffer : byte pointer to a buffer allocated to accept the frame
// lBuffSize : total buffer size in bytes, W * H * bytes per pixel
// iWaitms : block and wait iWaitms to get one image
// suggested wait is exposure * 2 + 500ms
// -1 means wait forever
//
// Return:
// ASI_SUCCESS
// ASI_ERROR_CAMERA_CLOSED : camera isn't open
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// ASI_ERROR_TIMEOUT : timeout, no image captured
// *****************************************************************************
function ASIGetVideoData(iCameraID: Integer; pBuffer: PBYTE; lBuffSize: Longint;
  iWaitms: Integer): Integer; cdecl; external ASICAMERA_API;

// *****************************************************************************
// ASIPulseGuideOn
// Description:
// Turn ST4 port pulse guider on and set direction
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
// direction : the direction of guider
//
// Return:
// ASI_SUCCESS
// ASI_ERROR_CAMERA_CLOSED : camera isn't open
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// *****************************************************************************
function ASIPulseGuideOn(iCameraID: Integer; direction: Integer): Integer;
  cdecl; external ASICAMERA_API;

// *****************************************************************************
// ASIPulseGuideOff
// Description:
// Turn ST4 port pulse guider off and set direction
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
// direction : the direction of guider
//
// Return:
// ASI_SUCCESS
// ASI_ERROR_CAMERA_CLOSED : camera isn't open
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// *****************************************************************************
function ASIPulseGuideOff(iCameraID: Integer; direction: Integer): Integer;
  cdecl; external ASICAMERA_API;

// *****************************************************************************
// ASIStartExposure
// Description:
// Start a single frame capture (snapshot)
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
//
// Return:
// ASI_SUCCESS
// ASI_ERROR_CAMERA_CLOSED : camera isn't open
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// *****************************************************************************
function ASIStartExposure(iCameraID: Integer): Integer; cdecl;
  external ASICAMERA_API;

// *****************************************************************************
// ASIStopExposure
// Description:
// Stop a single frame capture
// Can be used with ASIStartExposure for very long exposures without blocking
// Call ASIGetExpStatus to get exposure status after ASIStopExposure
// If status = ASI_SUCCESS, frame data is valid
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
//
// Return:
// ASI_SUCCESS
// ASI_ERROR_CAMERA_CLOSED : camera isn't open
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// *****************************************************************************
function ASIStopExposure(iCameraID: Integer): Integer; cdecl;
  external ASICAMERA_API;

// *****************************************************************************
// ASIGetExpStatus
// Description:
// Get exposure status after ASIStartExposure
// Can call continuously to check when frame is ready
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
// pExpStatus: Integer var parameter, ASI_EXPOSURE_STATUS value
//
// Return:
// ASI_SUCCESS
// ASI_ERROR_CAMERA_CLOSED : camera isn't open
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// *****************************************************************************
function ASIGetExpStatus(iCameraID: Integer; var pExpStatus: Integer): Integer;
  cdecl; external ASICAMERA_API;

// *****************************************************************************
// ASIGetDataAfterExp
// Description:
// Read frame after ASIGetExpStatus returns ASI_EXP_SUCCESS
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
// pBuffer : byte pointer to a buffer allocated to accept the frame
// lBuffSize : total buffer size in bytes
//
// Return:
// ASI_SUCCESS
// ASI_ERROR_CAMERA_CLOSED : camera isn't open
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// *****************************************************************************
function ASIGetDataAfterExp(iCameraID: Integer; pBuffer: PBYTE;
  lBuffSize: Longint): Integer; cdecl; external ASICAMERA_API;

// *****************************************************************************
// ASIGetID
// Description:
// Get ID string from camera ROM
// Only available with USB3 cameras
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
// pID : pointer to a TASI_ID structure to read the ID
//
// Return:
// ASI_SUCCESS
// ASI_ERROR_CAMERA_CLOSED : camera isn't open
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// *****************************************************************************
function ASIGetID(iCameraID: Integer; PID: PASI_ID): Integer; cdecl;
  external ASICAMERA_API;

// *****************************************************************************
// ASISetID
// Description:
// Write ID string to camera ROM
// Only available with USB3 cameras
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
// ID : TASI_ID structure with ID values to write
//
// Return:
// ASI_SUCCESS
// ASI_ERROR_CAMERA_CLOSED : camera isn't open
// ASI_ERROR_INVALID_ID : no camera connected or ID out of bounds
// *****************************************************************************
function ASISetID(iCameraID: Integer; ID: TASI_ID): Integer; cdecl;
  external ASICAMERA_API;

// *****************************************************************************
// ASIGetProductIDs
// Description:
// Get product IDs from camera
//
// Params:
// iCameraID : camera ID from TASICameraInfo structure
// pPIDs : pointer to a TASI_PID structure to receive the PIDs
//
// Return:
// Length of the PID array
// *****************************************************************************
function ASIGetProductIDs(pPIDs: PASI_PID): Integer; cdecl;
  external ASICAMERA_API;

implementation

initialization

IsMultiThread := TRUE;

end.
